<?php
namespace HCM\GSO\Logic;

use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\HTTP\Kernel;

class TabelaSalarialLogic {
    
    private $hcmParameters;
    
    public function __construct(Kernel $httpKernel, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->httpKernel    = $httpKernel;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function exportarTemplateSal($row) {
        try {
            $file = new \PHPExcel();
            $file->setActiveSheetIndex(0);
            $file->getActiveSheet()->setCellValue('A1', 'Descrição');
            $file->getActiveSheet()->setCellValue('B1', 'Cargo');
            $file->getActiveSheet()->setCellValue('C1', 'Nr Cargo');
            $file->getActiveSheet()->setCellValue('D1', 'Faixa');
            $file->getActiveSheet()->setCellValue('E1', 'Nr Faixa');
            for($i = 1; $i<=10; $i++){
                $file->getActiveSheet()->setCellValue(chr($i+69).'1', 'Nível '.$i);
            }
            $linha = 2;
            foreach($row["NMOCUPACAOH"] as $keyOcupa=>$nmOcupa){
                foreach($row["DSFAIXA"] as $keyFaixa=>$nmFaixa){
                    $file->getActiveSheet()->setCellValue('A'.$linha, $nmOcupa.' - '.$nmFaixa);
                    $file->getActiveSheet()->setCellValue('B'.$linha, $nmOcupa);
                    $file->getActiveSheet()->setCellValue('C'.$linha, $row["NROCUPACAOM"][$keyOcupa]);
                    $file->getActiveSheet()->setCellValue('D'.$linha, $nmFaixa);
                    $file->getActiveSheet()->setCellValue('E'.$linha, $row["NRFAIXA"][$keyFaixa]);
                    $linha++;
                }
            }
            return $this->download($file);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function download($file) {
        try {
            $nomeArquivo = "Exportacao_Salario.xls";
            $expDir = FPA::getTempDirectory();
            /*$expDir = 'APP_PATH' . '/uploads/' . $nrorg . '/Exportacao/';
            if (!file_exists($expDir)) {
                mkdir($expDir, 0777, true);
            }
            $expDir = realpath($expDir); */
            
            $write = \PHPExcel_IOFactory::createWriter($file, 'Excel5');
            $write->save($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);

            $file = ($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);
            return $file;
            //$file->
            //download($file);
        } catch (\Exception $e) {
            throw $e;
        }

    }
    
    public function importarTemplateSal($row){
        
        try {
            error_reporting(E_ERROR | E_PARSE);
            $arquivo = $row['DSPATH'][0];
            $this->entityManager->beginTransaction();
            $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory() . '/');
            if (!$file->isFile()) {
                throw new \Exception("Falha ao ler o arquivo.");
            }
            $caminho = $file->getPathname();

            $infos = \PHPExcel_IOFactory::load($caminho);
            $infos->setActiveSheetIndex(0);
            $idLinha = 2;
            $infoFinal= array();
            $nrestrutsind = isset($row['NRESTRUTSIND']) ? $row['NRESTRUTSIND'] : null;
            
            while(!empty($infos->getActiveSheet()->getCell('A'.$idLinha)->getValue())){
                $reg = array();
                $reg['DSTABELASALARIAL'] = $infos->getActiveSheet()->getCell('A'.$idLinha)->getValue();
                $reg['NROCUPACAOM']      = $infos->getActiveSheet()->getCell('C'.$idLinha)->getValue();
                $reg['NRFAIXA']          = $infos->getActiveSheet()->getCell('E'.$idLinha)->getValue();
                $reg['VR1SALTABELA']     = round($infos->getActiveSheet()->getCell('F'.$idLinha)->getValue(),2);
                $reg['VR2SALTABELA']     = round($infos->getActiveSheet()->getCell('G'.$idLinha)->getValue(),2);
                $reg['VR3SALTABELA']     = round($infos->getActiveSheet()->getCell('H'.$idLinha)->getValue(),2);
                $reg['VR4SALTABELA']     = round($infos->getActiveSheet()->getCell('I'.$idLinha)->getValue(),2);
                $reg['VR5SALTABELA']     = round($infos->getActiveSheet()->getCell('J'.$idLinha)->getValue(),2);
                $reg['VR6SALTABELA']     = round($infos->getActiveSheet()->getCell('K'.$idLinha)->getValue(),2);
                $reg['VR7SALTABELA']     = round($infos->getActiveSheet()->getCell('L'.$idLinha)->getValue(),2);
                $reg['VR8SALTABELA']     = round($infos->getActiveSheet()->getCell('M'.$idLinha)->getValue(),2);
                $reg['VR9SALTABELA']     = round($infos->getActiveSheet()->getCell('N'.$idLinha)->getValue(),2);
                $reg['VR10SALTABELA']    = round($infos->getActiveSheet()->getCell('O'.$idLinha)->getValue(),2);
    
                $idLinha++;
                $infoFinal[] = $reg;
            }
            foreach($infoFinal as $info){
                $this->insertTabelaSal($info, $row['DTINIVIGTABSALARIAL'], $row['DTBASETABSALARIAL'], $nrestrutsind);
            }
            
            $this->entityManager->flush();
            $this->entityManager->commit();
        
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    private function insertTabelaSal($info, $dtinivig, $dtbase, $nrestrutsind){
        try {
            $sql= $this->entityManager->getConnection()->prepare(
                "INSERT INTO GSO_TABELASALARIAL (
                    NRORG,
                    NRTABELASALARIAL,
                    NRFAIXA,
                    NROCUPACAOM,
                    DSTABELASALARIAL,
                    DTINIVIGTABSALARIAL,
                    DTFIMVIGTABSALARIAL,
                    DTBASETABSALARIAL,
                    VRPISOTABSALARIAL,
                    VRTETOTABSALARIAL,
                    VR1SALTABELA,
                    VR2SALTABELA,
                    VR3SALTABELA,
                    VR4SALTABELA,
                    VR5SALTABELA,
                    VR6SALTABELA,
                    VR7SALTABELA,
                    VR8SALTABELA,
                    VR9SALTABELA,
                    VR10SALTABELA,
                    IDATIVO,
                    DTINCLUSAO,
                    DTULTATU,
                    NRORGINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGULTATU,
                    CDOPERULTATU,
                    NRESTRUTSIND) VALUES (
                        :NRORG,
                        :NRTABELASALARIAL,
                        :NRFAIXA,
                        :NROCUPACAOM,
                        :DSTABELASALARIAL,
                        :DTINIVIGTABSALARIAL,
                        :DTFIMVIGTABSALARIAL,
                        :DTBASETABSALARIAL,
                        :VRPISOTABSALARIAL,
                        :VRTETOTABSALARIAL,
                        :VR1SALTABELA,
                        :VR2SALTABELA,
                        :VR3SALTABELA,
                        :VR4SALTABELA,
                        :VR5SALTABELA,
                        :VR6SALTABELA,
                        :VR7SALTABELA,
                        :VR8SALTABELA,
                        :VR9SALTABELA,
                        :VR10SALTABELA,
                        :IDATIVO,
                        :DTINCLUSAO,
                        :DTULTATU,
                        :NRORGINCLUSAO,
                        :CDOPERINCLUSAO,
                        :NRORGULTATU,
                        :CDOPERULTATU,
                        :NRESTRUTSIND)"    
            );
    
            $sql->execute(
                array(
                    "NRORG"                 =>  $this->hcmParameters->getNrorgTrabalho(),
                    "NRTABELASALARIAL"      =>  \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GSO_TABELASALARIAL', 1, 12, true),
                    "NRFAIXA"               =>  $info['NRFAIXA'],
                    "NROCUPACAOM"           =>  $info['NROCUPACAOM'],
                    "DSTABELASALARIAL"      =>  $info['DSTABELASALARIAL'],
                    "DTINIVIGTABSALARIAL"   =>  $dtinivig,
                    "DTFIMVIGTABSALARIAL"   =>  null,
                    "DTBASETABSALARIAL"     =>  $dtbase,
                    "VRPISOTABSALARIAL"     =>  null,
                    "VRTETOTABSALARIAL"     =>  null,
                    "VR1SALTABELA"          =>  $info['VR1SALTABELA'] === 0 ? null : $info['VR1SALTABELA'],
                    "VR2SALTABELA"          =>  $info['VR2SALTABELA'] === 0 ? null : $info['VR2SALTABELA'],
                    "VR3SALTABELA"          =>  $info['VR3SALTABELA'] === 0 ? null : $info['VR3SALTABELA'],
                    "VR4SALTABELA"          =>  $info['VR4SALTABELA'] === 0 ? null : $info['VR4SALTABELA'],
                    "VR5SALTABELA"          =>  $info['VR5SALTABELA'] === 0 ? null : $info['VR5SALTABELA'],
                    "VR6SALTABELA"          =>  $info['VR6SALTABELA'] === 0 ? null : $info['VR6SALTABELA'],
                    "VR7SALTABELA"          =>  $info['VR7SALTABELA'] === 0 ? null : $info['VR7SALTABELA'],
                    "VR8SALTABELA"          =>  $info['VR8SALTABELA'] === 0 ? null : $info['VR8SALTABELA'],
                    "VR9SALTABELA"          =>  $info['VR9SALTABELA'] === 0 ? null : $info['VR9SALTABELA'],
                    "VR10SALTABELA"         =>  $info['VR10SALTABELA'] === 0 ? null : $info['VR10SALTABELA'],
                    "IDATIVO"               =>  'S',
                    "DTINCLUSAO"            =>  DateUtil::getDataAtual()->format('d/m/Y h:m:s'),
                    "DTULTATU"              =>  DateUtil::getDataAtual()->format('d/m/Y h:m:s'),
                    "NRORGINCLUSAO"         =>  $this->hcmParameters->getNrorgTrabalho(),
                    "CDOPERINCLUSAO"        =>  $this->hcmParameters->getCdoperador(),
                    "NRORGULTATU"           =>  $this->hcmParameters->getNrorgTrabalho(),
                    "CDOPERULTATU"          =>  $this->hcmParameters->getCdoperador(),
                    "NRESTRUTSIND"          =>  $nrestrutsind
                )
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
}
