<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S1299Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getDadosS1299($parametros) {
        try {
            $dados = $this->getDadosPrincipais($parametros['NRESTRUTURAM'], $parametros['NRTABELA'], $parametros['NRTPAMBIENTE'], $parametros['TRANSDCTFWEB']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosPrincipais($nrestruturam, $nrtabela, $nrtpambiente, $dctfWeb) {
        $sql = <<<SQL
/*getDadosS1299*/
SELECT DISTINCT EMH.NRESTRUTURAM CHAVE,
       /*ideEvento*/
       DECODE(CF.NRTIPOMOVIMENT,1,1,10,1,19,1,4,2) as indApuracao,
       TO_CHAR(CF.DTOCORRENCIA, DECODE(CF.NRTIPOMOVIMENT,4,'YYYY','YYYY-MM')) as perApur,
       /*ideEmpregador*/
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       /*infoFech*/
       CASE WHEN NVL(ENV.S1200,0) + NVL(ENV.S2299,0) + NVL(ENV.S2399,0) > 0 THEN 'S' ELSE 'N' END evtRemun,
       CASE WHEN NVL(ENV.S1260,0) > 0 THEN 'S' ELSE 'N' END evtComProd,
       CASE WHEN NVL(ENV.S1270,0) > 0 THEN 'S' ELSE 'N' END evtContratAvNP,
       CASE WHEN NVL(ENV.S1280,0) > 0 THEN 'S' ELSE 'N' END evtInfoComplPer,
       NVL(:P_TRANSDCTFWEB, 'N')  transDCTFWeb
  FROM ESTRUTURAH EMH
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Movimento*/
  LEFT JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = EMH.NRORG
                               AND CF.NRTIPOMOVIMENT = :P_NRTABELA
                               AND CF.DTOCORRENCIA = :P_DTMESCOMPETENC
                               AND CF.NRTPMODALIDCAL = 1
                               AND CF.NROCORRECAL = 1
  /*Informações do Fechamento*/
  LEFT JOIN (SELECT PRIN.NRORG, PRIN.NRESTRUTURAM,
                    NVL(PRIN.S1200,0) S1200,
                    NVL(PRIN.S1260,0) S1260,
                    NVL(PRIN.S1270,0) S1270,
                    NVL(PRIN.S1280,0) S1280,
                    NVL(PRIN.S2299,0) S2299,
                    NVL(PRIN.S2399,0) S2399
               FROM (
                 SELECT * FROM (
                     SELECT ENV.NRORG, ENV.NRESTRUTURAM, EVE.NMEVENTO, COUNT(ENV.NRENVIO) QTDE
                       FROM ESO_ENVIOCADASTROS ENV
                       JOIN ESO_EVENTO EVE ON EVE.NREVENTO = ENV.NREVENTO
                      WHERE ENV.NRORG = :P_NRORG
                        AND ENV.NRESTRUTURAM = :P_NRESTRUTURAM
                        AND ENV.NRTPAMBIENTE = :P_NRTPAMBIENTE
                        AND TO_CHAR(ENV.DTMESCOMPETENC,'MM/YYYY') = TO_CHAR(TO_DATE(:P_DTMESCOMPETENC),'MM/YYYY')
                        AND ENV.IDATIVO = 'S'
                        AND (ENV.NRTABELA = :P_NRTABELA OR ENV.NRTABELA IS NULL)
                        AND ENV.NRENVIOEXCL IS NULL
                        AND (ENV.IDSTATUS = 'C' OR ENV.IDSTATUS = 'A')
                        AND (EVE.NMEVENTO IN ('S-1200','S-1260','S-1270','S-1280','S-2299','S-2399'))
                      GROUP BY ENV.NRORG, ENV.NRESTRUTURAM, EVE.NMEVENTO
                     UNION ALL
                     SELECT ENV.NRORG, ENV.NRESTRUTURAM, ENV.NMEVENTO, COUNT(ENV.NRENVIO) QTDE
                       FROM ESO_NAOPERIODICO ENV
                      WHERE ENV.NRORG = :P_NRORG
                        AND ENV.NRESTRUTURAM = :P_NRESTRUTURAM
                        AND ENV.NRTPAMBIENTE = :P_NRTPAMBIENTE
                        AND TO_CHAR(ENV.DTMESCOMPETENC,'MM/YYYY') = TO_CHAR(TO_DATE(:P_DTMESCOMPETENC),'MM/YYYY')
                        AND ENV.IDATIVO = 'S'
                        AND (ENV.NRTABELA = :P_NRTABELA OR ENV.NRTABELA IS NULL)
                        AND ENV.NRENVIOEXCL IS NULL
                        AND (ENV.IDSTATUS = 'C' OR ENV.IDSTATUS = 'A')
                        AND (ENV.NMEVENTO IN ('S-1200','S-1260','S-1270','S-1280','S-2299','S-2399'))
                      GROUP BY ENV.NRORG, ENV.NRESTRUTURAM, ENV.NMEVENTO
                     UNION ALL
                     SELECT ENV.NRORG, ENV.NRESTRUTURAM, ENV.NMEVENTO, COUNT(ENV.NRENVIO) QTDE
                       FROM ESO_PERIODICO ENV
                      WHERE ENV.NRORG = :P_NRORG
                        AND ENV.NRESTRUTURAM = :P_NRESTRUTURAM
                        AND ENV.NRTPAMBIENTE = :P_NRTPAMBIENTE
                        AND TO_CHAR(ENV.DTMESCOMPETENC,'MM/YYYY') = TO_CHAR(TO_DATE(:P_DTMESCOMPETENC),'MM/YYYY')
                        AND ENV.IDATIVO = 'S'
                        AND (ENV.NRTABELA = :P_NRTABELA OR ENV.NRTABELA IS NULL)
                        AND ENV.NRENVIOEXCL IS NULL
                        AND (ENV.IDSTATUS = 'C' OR ENV.IDSTATUS = 'A')
                        AND (ENV.NMEVENTO IN ('S-1200','S-1260','S-1270','S-1280','S-2299','S-2399'))
                      GROUP BY ENV.NRORG, ENV.NRESTRUTURAM, ENV.NMEVENTO
                 ) PIVOT (SUM(QTDE)
                     FOR NMEVENTO IN ('S-1200' AS S1200, 'S-1260' AS S1260, 'S-1270' AS S1270, 'S-1280' AS S1280, 'S-2299' AS S2299, 'S-2399' AS S2399))
               ) PRIN
             ) ENV ON ENV.NRORG = EMH.NRORG
                  AND ENV.NRESTRUTURAM = EMH.NRESTRUTURAM
 WHERE EMH.NRORG = :P_NRORG
   AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
   AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                AND ESTRUTURAH.NRORG = EMH.NRORG)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRTABELA" => $nrtabela,
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_TRANSDCTFWEB" => $dctfWeb
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

}