<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\SST\Services\HistoricoEpiService;

class HistoricoEpiController extends Crud {

    protected $dataSourceName = 'sst_historicoepi_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, HistoricoEpiService $historicoEpiService){
        
        $this->historicoEpiService = $historicoEpiService;
        $this->hcmParameters       = $hcmParameters;
        $this->parameterBag        = $parameterBag;
        
        parent::__construct($dataSourceManager);
    }

    public function getHistoricoEpi(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setHistoricoEpi(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_historicoepi";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRHISTORICOEPI', $row)) {
                    if ($row['NRHISTORICOEPI'] === '') {
                        $row['NRHISTORICOEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_HISTORICOEPI', 1, 12, true);
                    }
                } else {
                    $row['NRHISTORICOEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_HISTORICOEPI', 1, 12, true);
                }
                $row['NRORG']      = intval($row['NRORG']);
                $row['NRVINCULOM'] = intval($row['NRVINCULOM']);
                $this->historicoEpiService->validaCamposVazios($row);

            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setCadastroAutomaticoHistoricoEpi(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_historicoepi";
            $rows = array();
            
            foreach ($request->getDataSet()->getRows() as $row) {
                $currentRow = $row;
                
                // Auditoria e __is_new
                $currentRow['__is_new'] = true;
                $currentRow['DTINCLUSAO'] = $this->hcmParameters->getDataAtual();
                $currentRow['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
                $currentRow['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
                $currentRow['DTULTATU'] = $this->hcmParameters->getDataAtual();
                $currentRow['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
                $currentRow['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();

                $nrvinculos = $row['NRVINCULOM'];
                $nmvinculos = $row['NMVINCULOM'];
                $nrEPIs = $row['NREPI'];
                $nmEPIs = $row['NMEPI'];

                foreach($nrvinculos as $key => $nrvinculom) {
                    foreach($nrEPIs as $keyEPI => $epi) {
                        $currentRow['NRVINCULOM'] = $nrvinculom;
                        $currentRow['NMVINCULOM'] = $nmvinculos[$key];
                        $currentRow['NREPI'] = $epi;
                        $currentRow['NMEPI'] = $nmEPIs[$keyEPI];
                        $currentRow['NRHISTORICOEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_HISTORICOEPI', 1, 12, true);
                        $currentRow['NRORG']      = intval($row['NRORG']);
                        $currentRow['NRVINCULOM'] = intval($currentRow['NRVINCULOM']);
                        $this->historicoEpiService->validaCamposVazios($currentRow);
                        
                        $row = $currentRow;

                        parent::save($request, $response);
                    }
                }
            }

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteHistoricoEpi(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 