<?php

namespace HCM\VAB\IntegracaoVT;

use HCM\Util\Repositories;

use HCM\Util\DateUtil;

use Geral\Util\NovoCodigo;

use Teknisa\Model\Gestaopessoas\Entities\VabIntegtranspm;

use Zeedhi\Framework\DTO\Response\Message;

use Zeedhi\Framework\DTO\Response;

use Teknisa\Model\Gestaopessoas\Entities\VabIntegtransph;

use HCM\Util\FPA;

class Controlador {

    const OTIMO      = 10;
    const TRANSFACIL = 11;
    const DIRETO = 250;
    const FETRANSPOR = 1;

    private $tipoIntegracao;
    
    public static $qtdeRegistrosImportados;

    public function __construct( $tipoIntegracao,$hcmParameters,$entityManager)
    {
        $this->tipoIntegracao = $tipoIntegracao;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        self::$qtdeRegistrosImportados = 0;
    }
    
    public static function resetQtdeRegistrosImportados(){
        self::$qtdeRegistrosImportados = 0;
    }
    
    private function incrementaQtdeRegsitrosImportados(){
        self::$qtdeRegistrosImportados++;
    }
    
    public static function getQtdeRegistrosImportados(){
        return self::$qtdeRegistrosImportados;
    }
    
    private function insereIntegMestre( $infoIntegracao, $nrorg, $dtmescompetencia, $dtIniIntegracao, $dtFimIntegracao, $nrTipoLinhaVinculo = null)
    {
        try {
            $codigo = NovoCodigo::geraCodigo($nrorg, 'VAB_INTEGTRANSPM', 1, 21, true, true);
            $obj = new VabIntegtranspm();

            $obj->setNrintegtranspm($codigo);
            $obj->setDtinivalidadeinteg($dtIniIntegracao);
            $obj->setDtfinvalidadeinteg($dtFimIntegracao);
            $obj->setIdativo('S');
            $obj->setDtinclusao(DateUtil::getDataAtual());
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setCdoperinclusao(FPA::getCdoperador());
            $obj->setCdoperultatu(FPA::getCdOperador());
            $obj->setNrorg(FPA::getNrorgTrabalho());
            $obj->setNrorginclusao(FPA::getNrorgTrabalho());
            $obj->setNrorgultatu(FPA::getNrorgTrabalho());

            $this->entityManager->persist($obj);
            $this->entityManager->flush();
            $Message = $this->insereIntegHistor($codigo, $infoIntegracao, $nrorg, $dtmescompetencia, 'N', 0.01, 1, null, null, null, null, $nrTipoLinhaVinculo, true);
            return $Message;

        } catch (\Exception $e) {
            throw $e;
        }

    }

    private function insereIntegHistor( $nrMestre, $infoIntegracao, $nrorg, $dtmescompetencia, $reserva = 'N', $vrMultiplo = 0.01, $nrTipoIntegracao = 1,$cdformatointeg = null, $nrestruturam = null, $nrmatricula = null, $nrTipoLinhaVinculoAtual = null ,$nrTipoLinhaVinculo = null, $novoHistorico = false)
    {
        try {
            $codigo = NovoCodigo::geraCodigo($nrorg, 'VAB_INTEGTRANSPH', 1, 21, true, true);
            $Message = '';
            $obj = new VabIntegtransph();
            $obj->setNrintegtransph($codigo);
            $obj->setNrintegtranspm($nrMestre);
            $obj->setCdintegtransp($infoIntegracao['NUMERO_CARTAO']);
            if(is_numeric($infoIntegracao['SALDO_CARTAO']))
                $obj->setVrsaldoultconsulta($infoIntegracao['SALDO_CARTAO']);
            $obj->setDtmescompinteg($dtmescompetencia);
            $obj->setIdintegreserva($reserva);
            $obj->setIdativo('S');   
            $obj->setVrmultiplointeg($vrMultiplo);
            $obj->setDtultutilizacao(DateUtil::subtraiIntervalo(DateUtil::getDataAtual(true),2));
            $obj->setNrtipointegtransp($nrTipoIntegracao);
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setDtinclusao(DateUtil::getDataAtual());
            $obj->setCdoperinclusao(FPA::getCdOperador());
            $obj->setCdoperultatu(FPA::getCdOperador());
            $obj->setNrorg(FPA::getNrorgTrabalho());
            $obj->setNrorginclusao(FPA::getNrorgTrabalho());
            $obj->setNrorgultatu(FPA::getNrorgTrabalho());
            $this->setNrTipoLinhaVinculo($obj, $nrTipoLinhaVinculo, $novoHistorico, $nrTipoLinhaVinculoAtual);
            $Message = $this->setNrMatricula($obj, $infoIntegracao, $Message);

            $obj->setCdformatointeg($cdformatointeg);
            $obj->setNrestruturam($nrestruturam);

            $this->entityManager->persist($obj);
            $this->entityManager->flush();
            
            $this->incrementaQtdeRegsitrosImportados();

            return $Message;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function alteraIntegHistor($obj, $infoIntegracao, $nrTipoLinhaVinculo)
    {
        try {
            $Message = '';

            if(is_numeric($infoIntegracao['SALDO_CARTAO']))
                $obj->setVrsaldoultconsulta($infoIntegracao['SALDO_CARTAO']);

            $this->setNrTipoLinhaVinculo($obj, $nrTipoLinhaVinculo); 
            $Message = $this->setNrMatricula($obj, $infoIntegracao, $Message);
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setCdoperultatu(FPA::getCdOperador());
            $obj->setNrorgultatu(FPA::getNrorgTrabalho());
            $obj->setDtultutilizacao(DateUtil::subtraiIntervalo(DateUtil::getDataAtual(true),2));

            $this->entityManager->persist($obj);
            $this->entityManager->flush();
            
            $this->incrementaQtdeRegsitrosImportados();
            return $Message;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function setNrTipoLinhaVinculo($objeto, $nrTipoLinhaVinculo, $novoHistorico = false, $nrTipoLinhaVinculoAtual = null)
    {       
        if($novoHistorico == true || $this->tipoIntegracao != 1) {
            $nrTipoLinhaVinculo = !empty($nrTipoLinhaVinculo) ? $nrTipoLinhaVinculo : $this->tipoIntegracao;
            $objeto->setNrtipolinhavinc(intval($nrTipoLinhaVinculo));
            return;
        } else if ($this->tipoIntegracao == 1 ) {  
            if($objeto->getNrtipolinhavinc() === NULL) {
                $nrTipoLinhaVinculoHistorico = $nrTipoLinhaVinculoAtual;
            } else {
                $nrTipoLinhaVinculoHistorico = $objeto->getNrtipolinhavinc();
            }
            $nrTipoLinhaVinculo = !empty($nrTipoLinhaVinculo) ? $nrTipoLinhaVinculo : $nrTipoLinhaVinculoHistorico;
            $objeto->setNrtipolinhavinc($nrTipoLinhaVinculo);
            return;
        }
    }

    private function setNrMatricula($objeto, $infoIntegracao, $Message)
    {       
        if($this->tipoIntegracao == 1) {
            if(array_key_exists('MATRICULA', $infoIntegracao) && $infoIntegracao['MATRICULA']){
                $objeto->setNrmatricula($infoIntegracao['MATRICULA']);
                return;
            } else if($objeto->getNrmatricula() === NULL){
                $Message = 'O cartão '.$infoIntegracao['NUMERO_CARTAO'].' não possui matrícula registrada.';
                return $Message;
            }
        } else {
            if(array_key_exists('MATRICULA', $infoIntegracao) && $infoIntegracao['MATRICULA'] && is_numeric(intval($infoIntegracao['MATRICULA']))){
                $objeto->setNrmatricula(intval($infoIntegracao['MATRICULA']));
                return;
            }else if($objeto->getNrmatricula() === NULL){
                $Message = 'O cartão '.$infoIntegracao['NUMERO_CARTAO'].' não possui matrícula registrada.';
                return $Message;
            }
        }
    }

    public function executaIntegracao( $nrorg, $dtmescompetencia, $fileName, $caminhoArquivo, $dtIniIntegracao, $dtFimIntegracao, $message, $nrTipoLinhaVinculo)
    {
        try {
            switch ($this->tipoIntegracao) {
                case(self::OTIMO): {
                    $integracao = new LeitorOtimoII();
                    break;
                }
                case(self::TRANSFACIL): {
                    $integracao = new LeitorTransfacil();
                    break;
                }
                case(self::DIRETO): {
                    $integracao = new LeitorDireto();
                    break;
                }
                case(self::FETRANSPOR): {
                    $integracao = new LeitorFetranspor();
                    break;
                }
                default: {
                $integracao = '';
                }
            }

            $infos = $integracao->getInfo($caminhoArquivo);
            
            if(!$infos || (is_array($infos) && count($infos)==0) ) $message[]['MENSAGEM'] = 'A planilha "'.$fileName.'" não possui registros salvos.';
            
            foreach($infos as $info){
                $retornoIntergracao = $this->insereIntegração($info, $nrorg, $dtmescompetencia, $dtIniIntegracao, $dtFimIntegracao, $nrTipoLinhaVinculo);
                if($retornoIntergracao != ''){
                    $message[]['MENSAGEM'] = $retornoIntergracao;
                }
            }
            return $message;
        } catch (\Exception $e) {
            throw $e;
        }

    }


    private function buscaIntegracao( $nrorg , $codIntegracao, $dtmescompetencia)
    {
        try {
            $vabintegtransph = Repositories::VAB_INTEGTRANSPH;
            $sql = <<<SQL
            select vh from $vabintegtransph vh
            where
                vh.nrorg = :p_nrorg 
                and vh.nrintegtransph 
                in (
                    select it.nrintegtransph 
                    from $vabintegtransph it 
                    where it.nrorg = :p_nrorg 
                        and :p_cdintegtransp like CONCAT(it.cdintegtransp,'%') 
                        and it.dtmescompinteg = (
                            select max(m.dtmescompinteg) 
                            from $vabintegtransph m 
                            where m.cdintegtransp = it.cdintegtransp 
                            and m.nrorg = it.nrorg 
                            and m.dtmescompinteg <= :p_dtmescompetencia
                            ) 
                    )
            order by vh.dtmescompinteg desc
SQL;
            $params = array(
                'p_nrorg' => $nrorg,
                'p_cdintegtransp' => trim($codIntegracao),
                'p_dtmescompetencia' => $dtmescompetencia
            );
            $resultado = $this->entityManager->createQuery($sql)->execute($params);
            if (!empty($resultado)) {
                return $resultado[0];
            }
            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insereIntegração($info, $nrorg, $dtmescompetencia, $dtIniIntegracao, $dtFimIntegracao, $nrTipoLinhaVinculo = null){
        $Message = '';
        $ultHistorico = $this->buscaIntegracao($nrorg, $info['NUMERO_CARTAO'], $dtmescompetencia);
        
        if (!$ultHistorico) {
            $Message = $this->insereIntegMestre($info, $nrorg, $dtmescompetencia, $dtIniIntegracao, $dtFimIntegracao, $nrTipoLinhaVinculo);
        } else if($dtmescompetencia->format('d/m/Y') == $ultHistorico->getDtmescompinteg()->format('d/m/Y')) {
            $Message = $this->alteraIntegHistor($ultHistorico, $info, $nrTipoLinhaVinculo);
        }else{
            $Message = $this->insereIntegHistor($ultHistorico->getNrintegtranspm(), $info, $nrorg, $dtmescompetencia, $ultHistorico->getIdintegreserva(), $ultHistorico->getVrmultiplointeg(), $ultHistorico->getNrtipointegtransp(), $ultHistorico->getCdformatointeg(), $ultHistorico->getNrestruturam(), $ultHistorico->getNrmatricula(), $ultHistorico->getNrtipolinhavinc(), $nrTipoLinhaVinculo);
        }
        
        return $Message;
    }
}